@@ -9,8 +9,8 @@ my $temp = [];
 sub get {
     my ($size, $rules, $start) = @_;
     return unless ref $rules eq 'HASH'and ref $start eq 'HASH';
-    # return $temp->[$rules->{'nr'}] if exists $temp->[$rules->{'nr'}];
     my $transfer_function = $rules->{'f'};
+    my $action_function = $rules->{'action_f'};
     my $rule_size = $rules->{'size'};
     my @start = @{ $start->{'list'} };
     my $grid = [ [] ];
@@ -35,24 +35,40 @@ sub get {
         }
     }
     $grid->[0] = [splice @start, 0, $size_x];
+    my $agrid  = [ [(1) x $size_x] ];
     
     for my $row_i (1 .. $size_y - 1) { # compute next rows
         my $row = $grid->[$row_i] = [];
         my $brow = $grid->[$row_i-1];
-        my $val = $brow->[0];   # prerun for rule size
-        for my $cell_i (0 .. $size_x - $rule_size){
+        my $bact = $agrid->[$row_i-1];
+        my $val = $brow->[0];          # prerun for rule size
+        my $act = $agrid->[$row_i]  = [(0) x $size_x];
+        $val <<= 1;
+        $val += $brow->[1];
+        $val %= 8;
+        $row->[0]  = $bact->[0] ? $transfer_function->[ $val ] : $brow->[0];
+        $act->[0] |= 1 if $action_function->[ $val ] & 2;
+        $act->[1] |= 1 if $action_function->[ $val ] & 1;            
+        for my $cell_i (1 .. $size_x - 2){
             $val <<= 1;
             $val += $brow->[$cell_i+1];
             $val %= 8;
-            $row->[$cell_i] = $transfer_function->[ $val ];
+            $row->[$cell_i] = $bact->[$cell_i] ? $transfer_function->[ $val ] : $brow->[$cell_i];
+            $act->[$cell_i-1] |= 1 if $action_function->[ $val ] & 4;
+            $act->[$cell_i  ] |= 1 if $action_function->[ $val ] & 2;
+            $act->[$cell_i+1] |= 1 if $action_function->[ $val ] & 1;            
         }
-        for (1 .. $rule_size){
+        # for (1 .. $rule_size - 1)
+        {
             $val <<= 1; # last two elements are special
             $val %= 8;
-            $row->[$size_x - $rule_size + $_] = $transfer_function->[ $val ];
+            my $cell_i = $size_x - 1;
+            $row->[$cell_i] = $bact->[$cell_i] ? $transfer_function->[ $val ] : $brow->[$cell_i];
+            $act->[$cell_i-1] |= 1 if $action_function->[ $val ] & 4;
+            $act->[$cell_i  ] |= 1 if $action_function->[ $val ] & 2;
         }
     }
-    $temp->[$rules->{'nr'}] = $grid;
+    $grid;
 }
 
 1;
